@@ -4,12 +4,10 @@ package Data::Record::Serialize::Encode::null;
 
 use Moo::Role;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 use namespace::clean;
 
-has '+_need_types' => ( is => 'rwp', default => 0 );
-
 
 
 
@@ -52,7 +50,7 @@ Data::Record::Serialize::Encode::null - infinite bitbucket
 
 =head1 VERSION
 
-version 0.23
+version 0.24
 
 =head1 SYNOPSIS
 
@@ -79,7 +77,7 @@ L<Data::Record::Serialize::Role::Sink> roles.
 =head1 INTERFACE
 
 There are no additional attributes which may be passed to
-L<Data::Record::Serialize::new>|Data::Record::Serialize/new>.
+L<< Data::Record::Serialize::new|Data::Record::Serialize/new >>.
 
 =head1 SUPPORT
 
