@@ -4,7 +4,7 @@ package Data::Record::Serialize::Role::Default;
 
 use Moo::Role;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 use Hash::Util qw[ hv_store ];
 use Ref::Util qw[ is_coderef ];
@@ -34,7 +34,6 @@ use namespace::clean;
 
 # provide default if not already defined
 sub send {
-
     my $self = shift;
 
     $self->_needs_eol
@@ -42,11 +41,12 @@ sub send {
       : $self->print( $self->encode( @_ ) );
 }
 
-# provide default if not already defined
+# just in case they're not defined in preceding roles
 sub setup { }
+sub _map_types { }
+sub _needs_eol { 1 }
 
 around 'setup' => sub {
-
     my ( $orig, $self, $data ) = @_;
 
     # if fields has not been set yet, set it to the names in the data
@@ -57,17 +57,15 @@ around 'setup' => sub {
     my %dups;
     $dups{$_}++ && error( fields => "duplicate output field: $_" ) for@{$self->fields};
 
-    if ( $self->_need_types ) {
-
-        if ( $self->default_type ) {
-            $self->_set_types_from_default;
-        }
-        else {
-            $self->_set_types_from_record( $data );
-        }
-
-        $self->_set__need_types( 0 );
+    if ( $self->has_default_type ) {
+        $self->_set_types_from_default;
     }
+    else {
+        $self->_set_types_from_record( $data );
+    }
+
+    # trigger building of output_types, which also remaps types. ick.
+    $self->output_types;
 
     $orig->( $self );
 
@@ -76,7 +74,6 @@ around 'setup' => sub {
 
 
 before 'send' => sub {
-
     my ( $self, $data ) = @_;
 
     # can't do format or numify until we have types, which might need to
@@ -86,20 +83,26 @@ before 'send' => sub {
       if $self->_run_setup;
 
     # remove fields that won't be output
-    delete @{$data}{ grep { !defined $self->_fieldh->{$_} } keys %{$data} };
+    delete @{$data}{ grep { !exists $self->_fieldh->{$_} } keys %{$data} };
 
     # nullify fields (set to undef) those that are zero length
 
-    if ( defined( my $nullify = $self->_nullify ) ) {
-
+    if ( defined( my $fields = $self->_nullified ) ) {
         $data->{$_} = undef
-          for grep { defined $data->{$_} && !length $data->{$_} } @$nullify;
+          for grep { defined $data->{$_} && !length $data->{$_} } @$fields;
     }
 
-    if ( $self->_format ) {
+    if ( defined( my $fields = $self->_numified ) ) {
+        $data->{$_} = ( $data->{$_} || 0 ) + 0
+          for grep { defined $data->{$_} } @{$fields};
+    }
 
-        my $format = $self->_format;
+    if ( defined( my $fields = $self->_stringified ) ) {
+        $data->{$_} = "@{[ $data->{$_}]}"
+          for grep { defined $data->{$_} } @{$fields};
+    }
 
+    if ( my $format = $self->_format ) {
         $data->{$_}
           = is_coderef( $format->{$_} )
           ? $format->{$_}( $data->{$_} )
@@ -108,33 +111,35 @@ before 'send' => sub {
           keys %{$format};
     }
 
-    if ( $self->_numify ) {
 
-        $_ = ( $_ || 0 ) + 0 foreach @{$data}{ @{ $self->numeric_fields } };
+    # handle boolean
+    if ( $self->_boolify ) {
+        my @fields = grep { exists $data->{$_} } @{ $self->boolean_fields };
 
+        if ( $self->_can_bool ) {
+            $data->{$_} = $self->to_bool( $data->{$_} ) for @fields;
+        }
+
+        # the encoder doesn't have native boolean, must convert a
+        # truthy value to 0/1;
+        else {
+            $data->{$_} = $data->{$_} ? 1 : 0 foreach @fields;
+        }
     }
 
-    if ( $self->rename_fields ) {
-
-        my $rename = $self->rename_fields;
-
+    if ( my $rename = $self->rename_fields ) {
         for my $from ( @{ $self->fields } ) {
-
             my $to = $rename->{$from}
               or next;
 
             hv_store( %$data, $to, $data->{$from} );
             delete $data->{$from};
         }
-
     }
-
 };
 
 sub DEMOLISH {
-
     $_[0]->close;
-
     return;
 }
 
@@ -162,7 +167,7 @@ Data::Record::Serialize::Role::Default - Default methods for Data::Record::Seria
 
 =head1 VERSION
 
-version 0.23
+version 0.24
 
 =head1 DESCRIPTION
 
